package org.implab.gradle.mercurial.tasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.gradle.api.DefaultTask;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.Utils;
import org.implab.gradle.mercurial.MercurialExtension;

public class HgTask extends DefaultTask {

    private final Property<String> cliCmd;

    private List<Provider<String>> commandArgs;

    private String commandOutput;

    protected MercurialExtension getMercurialExtension() {
        return getProject().getExtensions().getByType(MercurialExtension.class);
    }

    public HgTask() {
        cliCmd = getProject().getObjects().property(String.class);
        cliCmd.convention(getMercurialExtension().getHgCmd());

    }

    public String getCommandOutput() {
        return commandOutput;
    }

    public void clearCommandArgs() {
        commandArgs.clear();
    }

    public void commandArgs(Object... args) {
        for (Object arg : args) {
            if (arg == null)
                continue;
            if (arg instanceof Provider<?>)
                commandArgs.add(((Provider<?>) arg).map(x -> x.toString()));
            else
                commandArgs.add(getProject().getProviders().provider(() -> arg.toString()));
        }
    }

    @TaskAction
    public void Run() throws IOException, InterruptedException, Exception {
        List<String> commandLine = new ArrayList<>();

        commandLine.add(cliCmd.get());

        for (Provider<String> arg : commandArgs) {
            if (arg.isPresent())
                commandLine.add(arg.get());
        }

        getLogger().info("Starting: ", commandLine);

        commandOutput = Utils.execCmd(commandLine, getMercurialExtension().getOutputCharset().name());
    }

}
